
static final int N = 0;
static final int NE = 1;
static final int E = 2;
static final int SE = 3;
static final int S = 4;
static final int SW = 5;
static final int W = 6;
static final int NW = 7;

World world;

void setup()
{
  //Applet size must be a mulitple for 30 for this to look right
  size(600,600);
  rectMode(CORNER);
  ellipseMode(CORNER);
  
  world = new World(width/30,height/30);
}

void draw()
{
  background(255);
  if(!world.goalReached)
  {
    if(world.move(world.navigate()))
    {
      world.draw();  
    }
    else
    {
      println("Invalid Move");
      exit();
    }
    
  }
}

public class Point2D
{
  int x;
  int y;
  
  Point2D(int _x, int _y)
  {
    x = _x;
    y = _y;
  }
}

public class Cell
{
  Point2D location;
  boolean isGoal;
  boolean isObstacle;
  boolean isNull;
  int cellSideLength = 30;
  
  Cell()
  {
    isGoal = isObstacle = false;
    location =  new Point2D(0,0);
  }
  
  void drawCell()
  {
    if(isObstacle)
    {
      fill(0);
    } 
    else if(isGoal)
    {
      fill(234,21,21);
    } 
    else 
    {
      noFill();
    }
    rect(this.location.x*cellSideLength, this.location.y*cellSideLength, cellSideLength, cellSideLength);
  }
  
  
}

public class World
{ 
  int _length;
  int _width;
  Point2D sailor;
  Point2D goal;
  Cell[][] cells;
  Cell[] localArea;
  Cell theNullCell = new Cell();
  boolean goalReached;
   
  World(int l, int w)
  {
    _length = l;
    _width = w;
    sailor = new Point2D(1,1);
    goal =  new Point2D(9,5);
    cells = new Cell[_length][_width];
    localArea = new Cell[8];
    theNullCell.isNull = true;

    goalReached = false;
   
    for(int k = 0; k<8; k++)
    {
      localArea[k] = new Cell();
    }
    for(int i = 0; i<_length; i++)
    {
      for(int j = 0; j<_width; j++)
      {
        cells[j][i] = new Cell();
        cells[j][i].location.x = j;
        cells[j][i].location.y = i;
      }
    }
    
    for(int n = 0; n <_length; n++)
    {
      cells[0][n].isObstacle = true;
      cells[_width-1][n].isObstacle = true;
    }
    
    for(int n = 0; n <_width; n++)
    {
      cells[n][0].isObstacle = true;
      cells[n][_length-1].isObstacle = true;
    }
    cells[4][3].isObstacle = true;
    cells[4][4].isObstacle = true;
    cells[4][5].isObstacle = true;
    cells[5][3].isObstacle = true;
    cells[5][4].isObstacle = true;
    cells[5][5].isObstacle = true;
    
    cells[6][8].isObstacle = true;
    cells[6][9].isObstacle = true;
    cells[7][8].isObstacle = true;
    cells[7][9].isObstacle = true;
    
    cells[9][5].isGoal = true;
    
  }
  
  void draw()
  {
    //May need to move this so that its done once
    for(int i = 0; i<_length; i++)
    {
      for(int j = 0; j<_width; j++)
      {
        cells[j][i].drawCell();
      }
    }
    fill(190); 
    ellipse(sailor.x*30,sailor.y*30,30,30);
   
  }
  
  int dirToGoal()
  {
    int goalHeading = 0;
    float temp_angle;
    
    temp_angle = atan2((goal.x - sailor.x), (goal.y - sailor.y));
    temp_angle += PI;
    temp_angle = 360*temp_angle/(2*PI);
    goalHeading = N;
    if (temp_angle > 22.5+0*45)
      goalHeading = NW;
    if (temp_angle > 22.5+1*45)
      goalHeading = W;
    if (temp_angle > 22.5+2*45)
      goalHeading = SW;
    if (temp_angle > 22.5+3*45)
      goalHeading = S;
    if (temp_angle > 22.5+4*45)
      goalHeading = SE;
    if (temp_angle > 22.5+5*45)
      goalHeading = E;
    if (temp_angle > 22.5+6*45)
      goalHeading = NE;
    if (temp_angle > 22.5+7*45)
      goalHeading = N;

    return goalHeading;
    //copy math from tuckers version
  }
  
  boolean move(int _heading)
  {
    boolean ret = false;
    Point2D newSailorPos = new Point2D(0,0);
    Point2D currentSailorPos = sailor;
    
    switch(_heading)
    {
      case N: 
        currentSailorPos.y--;
        break;
      case S: 
        currentSailorPos.y++;
        break;
      case E:
        currentSailorPos.x++;
        break;
      case W:
        currentSailorPos.x--;
        break;
      case NE:
        currentSailorPos.x++;
        currentSailorPos.y++;
        break;
      case NW:
        currentSailorPos.x++;
        currentSailorPos.y--;
        break;
      case SE:
        currentSailorPos.x--;
        currentSailorPos.y++;
        break;
      case SW:
        currentSailorPos.x--;
        currentSailorPos.y--;
        break;
      default:
        break;
    }
    //Check if heading isn't towards an obstacle or isn't towards bounds
    if(!((currentSailorPos.x > _length-1)) || currentSailorPos.x < 0) 
    {
      if(!((currentSailorPos.y > _width-1) || currentSailorPos.y < 0))
      {
        if(!(cells[newSailorPos.x][newSailorPos.y].isObstacle))
        {
          sailor = currentSailorPos;
          
          localArea[N] = cells[sailor.y - 1][sailor.x];
          localArea[NE] = cells[sailor.y - 1][sailor.x + 1];
          localArea[E] = cells[sailor.y][sailor.x + 1];
          localArea[SE] = cells[sailor.y + 1][sailor.x + 1];
          localArea[S] = cells[sailor.y + 1][sailor.x];
          localArea[SW] = cells[sailor.y + 1][sailor.x -1];
          localArea[W] = cells[sailor.y][sailor.x - 1];
          localArea[N] = cells[sailor.y - 1][sailor.x - 1];
          
          ret = true;
        }
        else if(cells[newSailorPos.x][newSailorPos.y].isGoal)
        {
          goalReached =  true;
          ret = true;
        }
      }
    }
    return ret;
  }
  
  int navigate()
  {
    int newHeading = 2;
    return newHeading;
  }
}

